from .c_parser_regex import extract_comments as iter_comments_sass
from .c_parser_state import iter_comments_c, iter_comments_go
from .css_parser_regex import extract_comments as iter_comments_css
from .html_parser_regex import extract_comments as iter_comments_html
from .python_parser import extract_comments as iter_comments_python
from .ruby_parser_regex import extract_comments as iter_comments_ruby
from .shell_parser_state import extract_comments as iter_comments_shell
